@@ -1,5 +1,5 @@
 """
-## calculate differential mec signature comparing two conditions
+## calculate differential mec signature comparing two conditions cluster-wise
 """
 import numpy as np
 import matplotlib.pyplot as plt
@@ -14,6 +14,7 @@ import scipy.stats as ss
 def _get_score_pd( adata, meccols ):
     """ 
     Obtain scores from scanpy object.
+    ----------
     compatible with:
         scanpy object with MetaTiME projection score in .X .
         or,
@@ -119,42 +120,30 @@ def enrich_mec( adata , labelcol , mecnamedict ):
     return(res1)
 
 def cohend( x1, x2 ):
-    """ t  test effect size """
+    """ t  test effect size , not used """
     cohens_d = (np.mean(x1) - np.mean(x2)) / (np.sqrt((np.std(x1) ** 2 + np.std(x2) ** 2) / 2))
     return(cohens_d)
 
-def test_sig_smallcluster(Xproj, meci, cellscond1, cellscond2):
+def test_sig_smallcluster(Xproj, meci, cellscond1, cellscond2, test_method ):
     """ 
-    testing signature difference at each cell group, for certain mec signatures in meci.
+    Testing signature difference at each cell group, for certain mec signatures in meci.
     """
     
     x1 = Xproj.loc[cellscond1, meci].values
     x2 = Xproj.loc[cellscond2, meci].values
 
-    #cut=2
-    #y1=x1[x1>=cut]
-    #y2=x2[x2>=cut]
-    #[s,p]=ss.ranksums(x2, x1)
-    [s,p]=ss.ttest_ind(x2, x1)
-    #u,p = ss.mannwhitneyu(x1,x2)
-    #logfc = np.log2((np.mean(y2)/len(x2))/(np.mean(y1)/len(x1)))# consider cell number differences
-    #logfc = np.log2((np.median(y2)+1)/(np.median(y1)+1))# consider cell number differences
-    #logfc = np.log2( (np.median(x2) +1 )/(np.median(x1) +1 ) )
+    if(test_method == 'ttest'):
+        [s,p]=ss.ttest_ind(x2, x1)
+    elif(test_method == 'wilcoxon'):
+        [s,p]=ss.ranksums(x2, x1)
     effect = (np.mean(x2)  - np.mean(x1) )  
-    
-    #logfc = cohend( x1, x2)
-    #logfc = np.log2((np.mean(x2))/(np.mean(x1)))
     meanb=np.mean(x2); meana=np.mean(x1)
-    #logctfc = np.log2((len(y2)/len(x2))/(len(y1)/len(x1)))# consider cell number differences
-    #logctfc = np.log2((np.sum(y2)/len(x2))/(np.sum(y1)/len(x1)))# consider cell number differences
-    #logctfc = np.log2( (np.sum(x2)/N2 +1)/(np.sum(x1)/N1 +1))# consider cell number differences
     effect1 = effect
-
     logp=-np.log10(p)
-    return( logp, effect,effect1,meanb,meana )
-### plot enriched 1st mec
+    return( logp, effect, effect1, meanb, meana )
 
 def plot_top1mec( adata, mec_enriched , legend_loc ='on data'):
+    """Plot enriched 1st mec"""
     mec_enriched['top1mec'] = mec_enriched['topmec'].str.split(',').str.get(0)
     adata.obs = adata.obs.merge( mec_enriched[['top1mec']], left_on = 'celltype_subset', right_index=True, how='left')
     sc.pl.umap(adata, color = ['top1mec'], legend_fontsize='5', legend_loc= legend_loc , alpha=0.4 )
@@ -167,7 +156,9 @@ def dmec( adata,
           clustercol: str  ,
           cluster_mec_enriched: pd.DataFrame, 
           mecnamedict,
-          test_clusters = None ,):
+          test_clusters = None ,
+          test_method: Union['ttest','wilcoxon'] = 'ttest',
+          ):
     """ 
     Differential signature analysis
 
@@ -190,7 +181,8 @@ def dmec( adata,
         can be loaded from pre-computed tumor MeC functional annotation
     test_clusters
         Which cell clusters to test based on adata.obs[clutsercol]
-
+    test_method
+        test statistics, either ttest or wilcoxon
     Returns
     ----------
     diffmec
@@ -222,7 +214,7 @@ def dmec( adata,
         res = pd.DataFrame(index=Xproj.columns)
         #N2 = len(allcellscond2);N1 = len(allcellscond1)
         for meci in Xproj.columns:
-            t = test_sig_smallcluster(Xproj, meci, cellscond1, cellscond2 )
+            t = test_sig_smallcluster(Xproj, meci, cellscond1, cellscond2, test_method=test_method )
             res.loc[meci, '-logp']= t[0]
             res.loc[meci, 'effect']= t[1]
             res.loc[meci, 'effect1']= t[2]
@@ -248,6 +240,7 @@ def dmec( adata,
 
 def plotdata_diffmec( diffmec, mecnamedict):
     """
+    Dot plot for differential mec signature
     """
     import seaborn as sns
     import matplotlib.pyplot as plt
@@ -275,6 +268,7 @@ def plotdata_diffmec( diffmec, mecnamedict):
 
 def plot_diffmec( diffmec, mecnamedict):
     """
+    
     """
     import seaborn as sns
     import matplotlib.pyplot as plt
@@ -325,10 +319,10 @@ def plot_diffmec( diffmec, mecnamedict):
 
 
 def topdiff_df(diffmec,
-             CUT_LOGPPOS = 1.3,
-             CUT_EFFECTPOS = 1,
-             CUT_EFFECTNEG = -1,
-             FCCOL = 'effect'
+             cut_logppos = 1.3,
+             cut_effectpos = 1,
+             cut_effectneg = -1,
+             effectkey = 'effect'
              ):
     """
     Organize per-cluster differential signature results into tables
@@ -337,14 +331,14 @@ def topdiff_df(diffmec,
     ----------
     diffmec
         differential expression dict generated by dmec.dmec
-    CUT_LOGPPOS 
+    cut_logppos 
         negative log p value cutoff to determin significance, default: 1.3 (#p=0.5)
-    CUT_EFFECTPOS
+    cut_effectpos
         positive effect size to cut . default: 1 ( 1 std shift)
-    CUT_EFFECTNEG
+    cut_effectneg
         negative effect size to cut . default: -1 ( 1 std shift)
-    FCCOL
-        the 'effect' to use (fold change)
+    effectkey
+        the 'effect' key to use in diffmec. 
 
     Example
     ----------
@@ -361,36 +355,48 @@ def topdiff_df(diffmec,
         df.index=df.index.str.replace( cluster1+'@'+cluster1 , cluster1 )
         tlst.append(df)
     dat = pd.concat(tlst)
-    sigpos = dat[( dat[ FCCOL ]>= CUT_EFFECTPOS ) & ( dat['-logp']>= CUT_LOGPPOS ) & ( (dat['meanb']>=1) )]
-    #sigpos = dat[( dat[ FCCOL ]>= CUT_EFFECTPOS ) & ( dat['-logp']>= CUT_LOGPPOS ) ]
-    signeg = dat[( dat[ FCCOL ]<= CUT_EFFECTNEG ) & ( dat['-logp']>= CUT_LOGPPOS ) & ( (dat['meana']>=1) )]
-    #signeg = dat[( dat[ FCCOL ]<= CUT_EFFECTNEG ) & ( dat['-logp']>= CUT_LOGPPOS ) ]
-    markpos = sigpos.sort_values(ascending=False, by= FCCOL ).index
-    markneg = signeg.sort_values(ascending=True, by= FCCOL ).index
+    sigpos = dat[( dat[ effectkey ]>= cut_effectpos ) & ( dat['-logp']>= cut_logppos ) & ( (dat['meanb']>=1) )]
+    #sigpos = dat[( dat[ effectkey ]>= cut_effectpos ) & ( dat['-logp']>= cut_logppos ) ]
+    signeg = dat[( dat[ effectkey ]<= cut_effectneg ) & ( dat['-logp']>= cut_logppos ) & ( (dat['meana']>=1) )]
+    #signeg = dat[( dat[ effectkey ]<= cut_effectneg ) & ( dat['-logp']>= cut_logppos ) ]
+    markpos = sigpos.sort_values(ascending=False, by= effectkey ).index
+    markneg = signeg.sort_values(ascending=True, by= effectkey ).index
     dat_sig = dat.loc[markpos.tolist() + markneg.tolist()].sort_values(ascending=False, by ='-logp')
     return( dat, dat_sig )
 
 
-def plot_topdiff( diffmec, fontsize = 5 ):
+def plot_topdiff( diffmec, fontsize = 5 ,
+                cut_logppos = 1.3, #p=0.5,
+                cut_effectpos = 1, # 1 std shift
+                cut_effectneg = -1, # 1 std shift
+                effectkey = 'effect', 
+                figsize=(9,6),
+                ):
     """
+    Plot top differential signature cluster-wise.
+    X-axis: Effect: difference of mean signature scores between two conditions. Y-axis: -⁡log(p⁡-value) from two-sided t test or wilcoxon test. 
+    For significant signatures, size of the dots is proportionally to the mean signature score in the high-signature group.
+
     Parameters
     ----------
     diffmec
-        dictonary containing per-cluster differential testing results
+        dictonary containing per-cluster differential testing results, usually generated by dmec.dmec
     fontsize
         fontsize on the differential scatter plot
-
+    effectkey
+        effect size column in diffmec
     Example
     ----------
     `sns.set_theme(style='white')
     fig_topdiff = dmec.plot_topdiff(diffmec, fontsize=6)
     `
     """
-    CUT_LOGPPOS = 1.3 #p=0.5
-    CUT_EFFECTPOS = 1 # 1 std shift
-    CUT_EFFECTNEG = -1 # 1 std shift
-    #FCCOL = 'logctft'
-    FCCOL = 'effect'
+    cut_logppos = cut_logppos
+    cut_effectpos = cut_effectpos
+    cut_effectneg = cut_effectneg
+    effectkey = effectkey
+
+    # The text to mark on the plot
     tlst = []
     for cluster1 in diffmec.keys():
         df = diffmec[ cluster1 ].copy()
@@ -402,34 +408,46 @@ def plot_topdiff( diffmec, fontsize = 5 ):
         tlst.append(df)
     dat = pd.concat(tlst)
 
-    fig,ax = plt.subplots(1, figsize=(6,5))
-    sns.set_theme(style="white")
-    sns.scatterplot(data=dat, x= FCCOL ,y='-logp', color='gray',legend=False,alpha=0.3)
-    sigpos = dat[( dat[ FCCOL ]>= CUT_EFFECTPOS ) & ( dat['-logp']>= CUT_LOGPPOS ) & ( (dat['meanb']>=1) )]
-    signeg = dat[( dat[ FCCOL ]<= CUT_EFFECTNEG ) & ( dat['-logp']>= CUT_LOGPPOS ) & ( (dat['meana']>=1) )]
-    markpos = sigpos.sort_values(ascending=False, by= FCCOL ).index
-    markneg = signeg.sort_values(ascending=True, by= FCCOL ).index
+    # Scatterplot for all enriched signature@cluster. size proportional to mean signature in the condition with higher signature.
+    fig,ax = plt.subplots(1, figsize = figsize)
+    sns.set_theme(style = "white")
+    
+    #scatter size and basic color
+    dat['Color'] = 'Sig.Unchanged'
+    dat['Sig. mean'] = 0
+    dat.loc[(dat[ effectkey ] < 0),'Sig.Score'] = dat.loc[(dat[ effectkey ] < 0),'meana']
+    dat.loc[(dat[ effectkey ] >= 0), 'Sig.Score'] = dat.loc[(dat[ effectkey ] >= 0),'meanb']
 
+    # Coloring significant ones
+    sigpos = dat[( dat[ effectkey ] >= cut_effectpos ) & ( dat['-logp']>= cut_logppos ) & ( (dat['meanb']>=1) )]
+    signeg = dat[( dat[ effectkey ] <= cut_effectneg ) & ( dat['-logp']>= cut_logppos ) & ( (dat['meana']>=1) )]
+    markpos = sigpos.sort_values(ascending=False, by= effectkey ).index
+    markneg = signeg.sort_values(ascending=True, by= effectkey ).index
+
+    dat.loc[markpos,'Color'] = 'Sig.Up'
+    dat.loc[markneg,'Color'] = 'Sig.Down'
+
+    # scatterplot
+    sns.scatterplot(data = dat, x = effectkey ,y = '-logp', hue = 'Color', palette={'Sig.Unchanged':'gray','Sig.Up':'brown','Sig.Down':'steelblue'},
+                    size='Sig.Score',legend=True,alpha=0.8, ax=ax)
+    ax.legend(bbox_to_anchor=(1.05, 1.0), loc='upper left')
+    plt.tight_layout()
     # tuning figure
     lim = np.max([abs(t) for t in ax.get_xlim()])
     ax.set_xlim([-lim,lim])
     #ax.set_ylim([-30, ax.get_ylim()[1]])
-    ax.axvline( CUT_EFFECTPOS ,linestyle='--', color='gray', alpha=0.4)
-    ax.axvline( CUT_EFFECTNEG,linestyle='--', color='gray', alpha=0.4)
+    ax.axvline( cut_effectpos ,linestyle='--', color='gray', alpha=0.4)
+    ax.axvline( cut_effectneg,linestyle='--', color='gray', alpha=0.4)
     ax.axvline( 0,linestyle='--', color='gray', alpha=0.6)
-    ax.axhline( CUT_LOGPPOS ,linestyle='--', color='gray', alpha=0.4)
-    # color
-    sns.scatterplot(data=sigpos, x= FCCOL ,y='-logp', color='brown', size='meanb',legend=False, ax=ax)
-    sns.scatterplot(data=signeg, x= FCCOL ,y='-logp', color='steelblue', size='meana',legend=False, ax=ax)
-    # text
+    ax.axhline( cut_logppos ,linestyle='--', color='gray', alpha=0.4)
+    
+    # Text annotate
     texts=[]
     for meci in markpos:
-        texts.append( plt.text( sigpos.loc[meci, FCCOL ],  sigpos.loc[meci,'-logp'], meci.replace('@','@') , fontsize= fontsize, color='red'))
+        texts.append( plt.text( sigpos.loc[meci, effectkey ],  sigpos.loc[meci,'-logp'], meci.replace('@','@') , fontsize= fontsize, color='red'))
     for meci in markneg:
-        texts.append( plt.text( signeg.loc[meci, FCCOL ],  signeg.loc[meci,'-logp'], meci.replace('@','@') , fontsize= fontsize, color='blue'))
-
+        texts.append( plt.text( signeg.loc[meci, effectkey ],  signeg.loc[meci,'-logp'], meci.replace('@','@') , fontsize= fontsize, color='blue'))
     from adjustText import adjust_text
     adjust_text(texts, arrowprops=dict(arrowstyle='-', color='gray'), autoalign='xy')
     ax.set_xlabel('Effect')
-    return( fig, ax )
-
+    return( fig, ax )
\ No newline at end of file
